function random() {
  return crypto.getRandomValues(new Uint32Array(2))[0];
}

function hex2bin(hex) {
  var arr = new Uint8Array(hex.length/2);
  for(var i = 0; i < hex.length/2; i++) {
    arr[i] = parseInt(hex.substr(2*i, 2), 16);
  }
  return arr;
}

function bin2str(arr) {
  return String.fromCharCode.apply(null, new Uint16Array(arr));
}

function str2bin(str) {
  var arr = new Uint16Array(str.length);
  for(var i = 0; i < str.length; i++) {
    arr[i] = str.charCodeAt(i);
  }
  return arr;
}

var sigKeyAlg = {
  name: "RSASSA-PKCS1-v1_5",
  params: {
    modulusLength: 512,
    publicExponent: new Uint8Array([0x01, 0x00, 0x01])
  }
};

var sigAlg = {
  name: "RSASSA-PKCS1-v1_5",
  params: { hash: "SHA-1" }
};

function signature(text) {
  var arr = str2bin(text);
  polycrypt.generateKey(sigKeyAlg).oncomplete = function(e) {
    var key = e.target.result;
    polycrypt.sign(sigAlg, key.privateKey, arr).oncomplete = function(e) {
      var sign= e.target.result;
      polycrypt.verify(sigAlg, key.publicKey, sign, arr).oncomplete = function(e) {
        console.log("verifyed?: "+e.target.result);
      };
    };
  };
};

var encRawKey = hex2bin("4ea1cb02f55c9bb9e795e647285d28cb20a49136296832c68a9048372ccfb2bf");

var encAlg = {
  name : "AES-GCM",
  params : {
    iv : hex2bin("534aea17"),
    additionalData: hex2bin("534aea17"),
    tagLength: 128
  }
};

function encrypt(text) {
  polycrypt.importKey("raw", encRawKey).oncomplete = function(e) {
    var key = e.target.result;
    polycrypt.encrypt(encAlg, key, str2bin(text)).oncomplete = function(e) {
      var cipher = e.target.result;
      polycrypt.decrypt(encAlg, key, cipher).oncomplete = function(e) {
		console.log("encrypted: "+bin2str(cipher));
        console.log("decrypted: "+bin2str(e.target.result));
      };
    };
  };
}

function createKey(params) {
  polycrypt.generateKey(params).oncomplete = function(e) {
    console.log(e.target.result);
  }
}